/* file_stat.c - display all info for specified files
**
** Copyright  2003,2005 by Jef Poskanzer <jef@mail.acme.com>.
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions
** are met:
** 1. Redistributions of source code must retain the above copyright
**    notice, this list of conditions and the following disclaimer.
** 2. Redistributions in binary form must reproduce the above copyright
**    notice, this list of conditions and the following disclaimer in the
**    documentation and/or other materials provided with the distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
** ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
** IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
** ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
** FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
** DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
** OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
** HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
** LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
** OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
** SUCH DAMAGE.
**
** For commentary on this license please see http://www.acme.com/license.html
*/


#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>


static char* modestr( mode_t mode );
static char* flagstr( u_int32_t flags );


int
main( int argc, char** argv )
    {
    char* argv0;
    int argn;
    int r;
    struct stat sb;
    char timebuf[100];
    const char* const timefmt = "%d%b%Y %H:%M:%S";

    argv0 = argv[0];

    if ( argc < 2 )
	{
	(void) fprintf( stderr, "usage: %s file ...\n", argv[0] );
	exit( 1 );
	}

    for ( argn = 1; argn < argc; ++argn )
	{
	if ( argn > 1 )
	    (void) printf( "\n" );
	r = lstat( argv[argn], &sb );
	if ( r != 0 )
	    {
	    perror( argv[argn] );
	    continue;
	    }
	(void) printf( "%s:\n", argv[argn] );
	(void) printf( "  st_dev      %d\n", (int) sb.st_dev );
	(void) printf( "  st_ino      %d\n", (int) sb.st_ino );
	(void) printf( "  st_mode     %011o  %s\n", (unsigned int) sb.st_mode, modestr( sb.st_mode ) );
	(void) printf( "  st_nlink    %d\n", (int) sb.st_nlink );
	(void) printf( "  st_uid      %d\n", (int) sb.st_uid );
	(void) printf( "  st_gid      %d\n", (int) sb.st_gid );
	(void) printf( "  st_rdev     %d\n", (int) sb.st_rdev );
	(void) strftime( timebuf, sizeof(timebuf), timefmt, localtime( &sb.st_atime ) );
	(void) printf( "  st_atime    %lu  %s\n", (unsigned long) sb.st_atime, timebuf );
	(void) strftime( timebuf, sizeof(timebuf), timefmt, localtime( &sb.st_mtime ) );
	(void) printf( "  st_mtime    %lu  %s\n", (unsigned long) sb.st_mtime, timebuf );
	(void) strftime( timebuf, sizeof(timebuf), timefmt, localtime( &sb.st_ctime ) );
	(void) printf( "  st_ctime    %lu  %s\n", (unsigned long) sb.st_ctime, timebuf );
	(void) printf( "  st_size     %lld\n", (long long) sb.st_size );
	(void) printf( "  st_blocks   %lld\n", (long long) sb.st_blocks );
	(void) printf( "  st_blksize  %ld\n", (long) sb.st_blksize );
	(void) printf( "  st_flags    0x%08lx  %s\n", (long) sb.st_flags, flagstr( sb.st_flags ) );
	(void) printf( "  st_gen      %ld\n", (long) sb.st_gen );
	}

    exit( 0 );
    }


static char*
modestr( mode_t mode )
    {
    static char str[20];

    /* First the file type. */
    switch ( mode & S_IFMT )
	{
	case S_IFIFO:  str[0] = 'p'; break;
	case S_IFCHR:  str[0] = 'c'; break;
	case S_IFDIR:  str[0] = 'd'; break;
	case S_IFBLK:  str[0] = 'b'; break;
	case S_IFREG:  str[0] = '-'; break;
	case S_IFLNK:  str[0] = 'l'; break;
	case S_IFSOCK: str[0] = 's'; break;
	case S_IFWHT:  str[0] = 'w'; break;	/* A "whiteout"?  WTF? */
	default:       str[0] = '?'; break;
	}

    /* Now the owner, group, and world permissions. */
    str[1] = ( mode & S_IRUSR ) ? 'r' : '-';
    str[2] = ( mode & S_IWUSR ) ? 'w' : '-';
    if ( mode & S_ISUID )
	str[3] = ( mode & S_IXUSR ) ? 's' : 'S';
    else
	str[3] = ( mode & S_IXUSR ) ? 'x' : '-';
    str[4] = ( mode & S_IRGRP ) ? 'r' : '-';
    str[5] = ( mode & S_IWGRP ) ? 'w' : '-';
    if ( mode & S_ISGID )
	str[6] = ( mode & S_IXGRP ) ? 's' : 'S';
    else
	str[6] = ( mode & S_IXGRP ) ? 'x' : '-';
    str[7] = ( mode & S_IROTH ) ? 'r' : '-';
    str[8] = ( mode & S_IWOTH ) ? 'w' : '-';
    if ( mode & S_ISVTX )
	str[9] = ( mode & S_IXOTH ) ? 't' : 'T';
    else
	str[9] = ( mode & S_IXOTH ) ? 'x' : '-';
    str[10] = '\0';

    return str;
    }


static char*
flagstr( u_int32_t flags )
    {
    static char str[500];

    str[0] = '\0';

    if ( flags & SF_ARCHIVED )
	(void) strcat( str, "arch " );
    if ( flags & SF_IMMUTABLE )
	(void) strcat( str, "schg " );
    if ( flags & SF_APPEND )
	(void) strcat( str, "sappnd " );
    if ( flags & SF_NOUNLINK )
	(void) strcat( str, "sunlnk " );
    if ( flags & UF_NODUMP )
	(void) strcat( str, "nodump " );
    if ( flags & UF_IMMUTABLE )
	(void) strcat( str, "uchg " );
    if ( flags & UF_APPEND )
	(void) strcat( str, "uappnd " );
    if ( flags & UF_OPAQUE )
	(void) strcat( str, "opaque " );
    if ( flags & UF_NOUNLINK )
	(void) strcat( str, "uunlnk " );

    if ( str[strlen( str ) - 1] == ' ' )
	str[strlen( str ) - 1] = '\0';

    return str;
    }
